.container {
  width: 80%;
}
.navbar1 {
  padding-bottom: 10px;
}
.navbar1 .nav-pills {
  margin-bottom: 0px;
  font-size: 12px;
}
.navbar1 .nav-pills > li > a {
  padding-top: 2.5px;
  padding-bottom: 2.5px;
}
.navbar1 .dropdown.dropdown-menu {
	padding: 0px;
}
.header {
  padding-top: 30px;
  padding-bottom: 18px;
}
.SearchBar .form-search {
  margin-bottom: 0px;
}
.SearchBar .input-append input {
  height: 12px;
}
body {
  font-family: Segoe UI;
  background-color: #f9faf7;
}
.main {
  background-color: white;
  padding: 18px;
  -moz-box-shadow: 0 0 3px #888;
  -webkit-box-shadow: 0 0 3px #888;
  box-shadow: 0 0 3px #888;
}
.MainHeader h1 {
  font-weight: normal;
}
.content .contentTitle h4 {
  font-size: 18px;
  font-weight: normal;
}
.content .meta {
  font-size: small;
}
.tags .btn {
  border: none;
  font-size: 10px;
  font-weight: bold;
}
.navigation {
  font-size: 12px;
  padding-bottom: 12px;
}
.navigation .nav-title {
  color: #333333;
  font-family: "Segoe UI semibold";
  font-size: 16px;
  text-transform: uppercase;
}
.navigation li {
  margin: 5px;
}
.snippetHeader {
  margin-bottom: 5px;
}
.snippetHeader .snippetTitle {
  font-size: 21px;
  line-height: 40px;
  border-bottom: 1px solid #e5e5e5;
  display: block;
  color: #333333;
}
.snippetInfo {
  padding-top: 10px;
}
.snippetInfo .dl-horizontal {
  margin: 5px;
}
.snippet-body {
  padding: 10px;
}
.snippet-body .accordion-group {
  border: none;
}
.snippet-body .accordion-heading {
  text-transform: uppercase;
  font-size: 14px;
  border-bottom: 1px solid #e5e5e5;
}
.snippet-body .accordion-heading .accordion-toggle {
  padding-top: 10px;
  padding-bottom: 5px;
}
.SearchResult {
  padding: 10px;
  padding-top: 0px;
}
.SearchResult .PageTitle {
  font-size: 21px;
  line-height: 40px;
  border-bottom: 1px solid #e5e5e5;
  padding-bottom: 5px;
  display: block;
  color: #333333;
}
.footer {
  padding: 10px;
}
.footer-inside {
  border-top: 1px solid #e5e5e5;
  padding: 10px;
}
